@@ -10,6 +10,7 @@ import os  # load the python os default module
 import sys  # laod the python sys default module
 import copy
 import re
+import time
 import warnings
 
 import numpy as np
@@ -79,11 +80,9 @@ class PandaPowerBackend(Backend):
     v_ex: :class:`numpy.array`, dtype:float
         The voltage magnitude at the extremity bus of the powerline
 
-    thermal_limit: :class:`numpy.array`, dtype:float
-        The
     """
-    def __init__(self):
-        Backend.__init__(self)
+    def __init__(self, detailed_infos_for_cascading_failures=False):
+        Backend.__init__(self, detailed_infos_for_cascading_failures=detailed_infos_for_cascading_failures)
         self.prod_pu_to_kv = None
         self.load_pu_to_kv = None
         self.lines_or_pu_to_kv = None
@@ -98,6 +97,14 @@ class PandaPowerBackend(Backend):
         self.v_ex = None
         self.a_ex = None
 
+        self.load_p = None
+        self.load_q = None
+        self.load_v = None
+
+        self.prod_p = None
+        self.prod_q = None
+        self.prod_v = None
+
         self._pf_init = "flat"
         self._pf_init = "results"
         self._nb_bus_before = 0
@@ -107,14 +114,14 @@ class PandaPowerBackend(Backend):
         self._iref_slack = None
         self._id_bus_added = None
         self._fact_mult_gen = -1
-        self._subs_to_table = None
         self._what_object_where = None
         self._number_true_line = -1
         self._corresp_name_fun = {}
         self._get_vector_inj = {}
-        self._dim_topo = -1
+        self.dim_topo = -1
         self._vars_action = Action.vars_action
         self._vars_action_set = Action.vars_action_set
+        # self._time_topo_vect = 0.
 
     def get_nb_active_bus(self):
         """
@@ -148,7 +155,10 @@ class PandaPowerBackend(Backend):
         if not os.path.exists(full_path):
             raise RuntimeError("There is no powergrid at \"{}\"".format(full_path))
 
-        self._grid = pp.from_json(full_path)
+        with warnings.catch_warnings():
+            # remove deprecationg warnings for old version of pandapower
+            warnings.filterwarnings("ignore", category=DeprecationWarning)
+            self._grid = pp.from_json(full_path)
 
         # add the slack bus that is often not modeled as a generator, but i need it for this backend to work
         bus_gen_added = None
@@ -189,45 +199,44 @@ class PandaPowerBackend(Backend):
         # self._fact_mult_gen[-1] += 1
 
         # now extract the powergrid
-        self.n_lines = copy.deepcopy(self._grid.line.shape[0]) + copy.deepcopy(self._grid.trafo.shape[0])
-        self.name_lines = ['{from_bus}_{to_bus}_{id_powerline_me}'.format(**row, id_powerline_me=i)
-                           for i, (_, row) in enumerate(self._grid.line.iterrows())]
+        self.n_line = copy.deepcopy(self._grid.line.shape[0]) + copy.deepcopy(self._grid.trafo.shape[0])
+        self.name_line = ['{from_bus}_{to_bus}_{id_powerline_me}'.format(**row, id_powerline_me=i)
+                          for i, (_, row) in enumerate(self._grid.line.iterrows())]
         transfo =  [('{hv_bus}'.format(**row), '{lv_bus}'.format(**row))
                     for i, (_, row) in enumerate(self._grid.trafo.iterrows())]
         transfo =  [sorted(el) for el in transfo]
-        self.name_lines += ['{}_{}_{}'.format(*el, i+self._grid.line.shape[0]) for i, el in enumerate(transfo)]
-        self.name_lines = np.array(self.name_lines)
+        self.name_line += ['{}_{}_{}'.format(*el, i + self._grid.line.shape[0]) for i, el in enumerate(transfo)]
+        self.name_line = np.array(self.name_line)
 
-        self.n_generators = copy.deepcopy(self._grid.gen.shape[0])
-        self.name_prods = ["gen_{bus}_{index_gen}".format(**row, index_gen=i)
-                           for i, (_, row) in enumerate(self._grid.gen.iterrows())]
-        self.name_prods = np.array(self.name_prods)
+        self.n_gen = copy.deepcopy(self._grid.gen.shape[0])
+        self.name_gen = ["gen_{bus}_{index_gen}".format(**row, index_gen=i)
+                         for i, (_, row) in enumerate(self._grid.gen.iterrows())]
+        self.name_gen = np.array(self.name_gen)
 
-        self.n_loads = copy.deepcopy(self._grid.load.shape[0])
-        self.name_loads = ["load_{bus}_{index_gen}".format(**row, index_gen=i)
-                           for i, (_, row) in enumerate(self._grid.load.iterrows())]
-        self.name_loads = np.array(self.name_loads)
+        self.n_load = copy.deepcopy(self._grid.load.shape[0])
+        self.name_load = ["load_{bus}_{index_gen}".format(**row, index_gen=i)
+                          for i, (_, row) in enumerate(self._grid.load.iterrows())]
+        self.name_load = np.array(self.name_load)
 
-        self.n_substations = copy.deepcopy(self._grid.bus.shape[0])
-        self.name_subs = ["sub_{}".format(i) for i, row in self._grid.bus.iterrows()]
-        self.name_subs = np.array(self.name_subs)
+        self.n_sub = copy.deepcopy(self._grid.bus.shape[0])
+        self.name_sub = ["sub_{}".format(i) for i, row in self._grid.bus.iterrows()]
+        self.name_sub = np.array(self.name_sub)
 
         #  number of elements per substation
-        self.subs_elements = np.zeros(self.n_substations, dtype=np.int)
+        self.sub_info = np.zeros(self.n_sub, dtype=np.int)
 
-        self.load_to_subid = np.zeros(self.n_loads, dtype=np.int)
-        self.gen_to_subid = np.zeros(self.n_generators, dtype=np.int)
-        self.lines_or_to_subid = np.zeros(self.n_lines, dtype=np.int)
-        self.lines_ex_to_subid = np.zeros(self.n_lines, dtype=np.int)
+        self.load_to_subid = np.zeros(self.n_load, dtype=np.int)
+        self.gen_to_subid = np.zeros(self.n_gen, dtype=np.int)
+        self.line_or_to_subid = np.zeros(self.n_line, dtype=np.int)
+        self.line_ex_to_subid = np.zeros(self.n_line, dtype=np.int)
 
-        self.load_to_sub_pos = np.zeros(self.n_loads, dtype=np.int)
-        self.gen_to_sub_pos = np.zeros(self.n_generators, dtype=np.int)
-        self.lines_or_to_sub_pos = np.zeros(self.n_lines, dtype=np.int)
-        self.lines_ex_to_sub_pos = np.zeros(self.n_lines, dtype=np.int)
+        self.load_to_sub_pos = np.zeros(self.n_load, dtype=np.int)
+        self.gen_to_sub_pos = np.zeros(self.n_gen, dtype=np.int)
+        self.line_or_to_sub_pos = np.zeros(self.n_line, dtype=np.int)
+        self.line_ex_to_sub_pos = np.zeros(self.n_line, dtype=np.int)
 
-        self._subs_to_table = [None for _ in range(self.n_substations)] # TODO later if I have too
-        pos_already_used = np.zeros(self.n_substations, dtype=np.int)
-        self._what_object_where = [[] for _ in range(self.n_substations)]
+        pos_already_used = np.zeros(self.n_sub, dtype=np.int)
+        self._what_object_where = [[] for _ in range(self.n_sub)]
 
         # self._grid.line.sort_index(inplace=True)
         # self._grid.trafo.sort_index(inplace=True)
@@ -237,14 +246,14 @@ class PandaPowerBackend(Backend):
         for i, (_, row) in enumerate(self._grid.line.iterrows()):
             sub_or_id = int(row["from_bus"])
             sub_ex_id = int(row["to_bus"])
-            self.subs_elements[sub_or_id] += 1
-            self.subs_elements[sub_ex_id] += 1
-            self.lines_or_to_subid[i] = sub_or_id
-            self.lines_ex_to_subid[i] = sub_ex_id
+            self.sub_info[sub_or_id] += 1
+            self.sub_info[sub_ex_id] += 1
+            self.line_or_to_subid[i] = sub_or_id
+            self.line_ex_to_subid[i] = sub_ex_id
 
-            self.lines_or_to_sub_pos[i] = pos_already_used[sub_or_id]
+            self.line_or_to_sub_pos[i] = pos_already_used[sub_or_id]
             pos_already_used[sub_or_id] += 1
-            self.lines_ex_to_sub_pos[i] = pos_already_used[sub_ex_id]
+            self.line_ex_to_sub_pos[i] = pos_already_used[sub_ex_id]
             pos_already_used[sub_ex_id] += 1
 
             self._what_object_where[sub_or_id].append(("line", "from_bus", i))
@@ -255,14 +264,14 @@ class PandaPowerBackend(Backend):
         for i, (_, row) in enumerate(self._grid.trafo.iterrows()):
             sub_or_id = int(row["hv_bus"])
             sub_ex_id = int(row["lv_bus"])
-            self.subs_elements[sub_or_id] += 1
-            self.subs_elements[sub_ex_id] += 1
-            self.lines_or_to_subid[i+lag_transfo] = sub_or_id
-            self.lines_ex_to_subid[i+lag_transfo] = sub_ex_id
+            self.sub_info[sub_or_id] += 1
+            self.sub_info[sub_ex_id] += 1
+            self.line_or_to_subid[i + lag_transfo] = sub_or_id
+            self.line_ex_to_subid[i + lag_transfo] = sub_ex_id
 
-            self.lines_or_to_sub_pos[i+lag_transfo] = pos_already_used[sub_or_id]
+            self.line_or_to_sub_pos[i + lag_transfo] = pos_already_used[sub_or_id]
             pos_already_used[sub_or_id] += 1
-            self.lines_ex_to_sub_pos[i+lag_transfo] = pos_already_used[sub_ex_id]
+            self.line_ex_to_sub_pos[i + lag_transfo] = pos_already_used[sub_ex_id]
             pos_already_used[sub_ex_id] += 1
 
             self._what_object_where[sub_or_id].append(("trafo", "hv_bus", i))
@@ -270,7 +279,7 @@ class PandaPowerBackend(Backend):
 
         for i, (_, row) in enumerate(self._grid.gen.iterrows()):
             sub_id = int(row["bus"])
-            self.subs_elements[sub_id] += 1
+            self.sub_info[sub_id] += 1
             self.gen_to_subid[i] = sub_id
             self.gen_to_sub_pos[i] = pos_already_used[sub_id]
             pos_already_used[sub_id] += 1
@@ -279,7 +288,7 @@ class PandaPowerBackend(Backend):
 
         for i, (_, row)  in enumerate(self._grid.load.iterrows()):
             sub_id = int(row["bus"])
-            self.subs_elements[sub_id] += 1
+            self.sub_info[sub_id] += 1
             self.load_to_subid[i] = sub_id
             self.load_to_sub_pos[i] = pos_already_used[sub_id]
             pos_already_used[sub_id] += 1
@@ -288,8 +297,7 @@ class PandaPowerBackend(Backend):
 
         self._compute_pos_big_topo()
 
-        self._dim_topo = np.sum(self.subs_elements)
-
+        self.dim_topo = np.sum(self.sub_info)
 
         # utilities for imeplementing apply_action
         self._corresp_name_fun = {}
@@ -306,25 +314,24 @@ class PandaPowerBackend(Backend):
         add_topo["in_service"] = False
         self._grid.bus = pd.concat((self._grid.bus, add_topo))
 
-        self.prod_pu_to_kv = self._grid.bus["vn_kv"][self._grid.gen["bus"]].values
-        self.load_pu_to_kv = self._grid.bus["vn_kv"].values[self._grid.load["bus"]]
-        self.lines_or_pu_to_kv  = np.concatenate((self._grid.bus["vn_kv"][self._grid.line["from_bus"].values],
-                                self._grid.bus["vn_kv"][self._grid.trafo["hv_bus"].values]))
-        self.lines_ex_pu_to_kv = np.concatenate((self._grid.bus["vn_kv"][self._grid.line["to_bus"].values],
-                                self._grid.bus["vn_kv"][self._grid.trafo["lv_bus"].values]))
+        self.load_pu_to_kv = self._grid.bus["vn_kv"][self.load_to_subid].values
+        self.prod_pu_to_kv  = self._grid.bus["vn_kv"][self.gen_to_subid].values
+        self.lines_or_pu_to_kv = self._grid.bus["vn_kv"][self.line_or_to_subid].values
+        self.lines_ex_pu_to_kv = self._grid.bus["vn_kv"][self.line_ex_to_subid].values
+
         self._nb_bus_before = self.get_nb_active_bus()
 
         self.thermal_limit_a = 1000 * np.concatenate((self._grid.line["max_i_ka"].values,
                                                       self._grid.trafo["sn_mva"].values / (np.sqrt(3) * self._grid.trafo["vn_hv_kv"].values)))
 
-        self.p_or = np.full(self.n_lines, dtype=np.float, fill_value=np.NaN)
-        self.q_or = np.full(self.n_lines, dtype=np.float, fill_value=np.NaN)
-        self.v_or = np.full(self.n_lines, dtype=np.float, fill_value=np.NaN)
-        self.a_or = np.full(self.n_lines, dtype=np.float, fill_value=np.NaN)
-        self.p_ex = np.full(self.n_lines, dtype=np.float, fill_value=np.NaN)
-        self.q_ex = np.full(self.n_lines, dtype=np.float, fill_value=np.NaN)
-        self.v_ex = np.full(self.n_lines, dtype=np.float, fill_value=np.NaN)
-        self.a_ex = np.full(self.n_lines, dtype=np.float, fill_value=np.NaN)
+        self.p_or = np.full(self.n_line, dtype=np.float, fill_value=np.NaN)
+        self.q_or = np.full(self.n_line, dtype=np.float, fill_value=np.NaN)
+        self.v_or = np.full(self.n_line, dtype=np.float, fill_value=np.NaN)
+        self.a_or = np.full(self.n_line, dtype=np.float, fill_value=np.NaN)
+        self.p_ex = np.full(self.n_line, dtype=np.float, fill_value=np.NaN)
+        self.q_ex = np.full(self.n_line, dtype=np.float, fill_value=np.NaN)
+        self.v_ex = np.full(self.n_line, dtype=np.float, fill_value=np.NaN)
+        self.a_ex = np.full(self.n_line, dtype=np.float, fill_value=np.NaN)
         self._nb_bus_before = None
 
     def apply_action(self, action: Action):
@@ -333,41 +340,40 @@ class PandaPowerBackend(Backend):
         """
 
         if not isinstance(action, Action):
-            raise UnrecognizedAction("Action given to PandaPowerBackend should be of class Action and not \"{}\"".format(action.__class__))
+            raise UnrecognizedAction("Action given to PandaPowerBackend should be of class Action and not "
+                                     "\"{}\"".format(action.__class__))
 
         # change the _injection if needed
-        dict_injection, change_status, switch_status, set_topo_vect, switcth_topo_vect = action()
+        dict_injection, set_status, switch_status, set_topo_vect, switcth_topo_vect, redispatching = action()
+
         for k in dict_injection:
             if k in self._vars_action_set:
                 tmp = self._get_vector_inj[k](self._grid)
-                val = dict_injection[k]
+                val = 1. * dict_injection[k]
                 ok_ind = np.isfinite(val)
                 if k == "prod_v":
                     pass
-                    # continue
                     # convert values back to pu
-                    val /= self.prod_pu_to_kv # self._grid.bus["vn_kv"][self._grid.gen["bus"]].values
-                    # try:
+                    val /= self.prod_pu_to_kv  # self._grid.bus["vn_kv"][self._grid.gen["bus"]].values
                     if self._id_bus_added is not None:
                         # in this case the slack bus where not modeled as an independant generator in the
                         # original data
                         if np.isfinite(val[self._id_bus_added]):
                             # handling of the slack bus, where "2" generators are present.
                             pass
-                            # self._grid._ppc['gen'][self._iref_slack, 5] = val[self._id_bus_added]
-                            # self._grid._ppc['gen'][-1, 5] = val[self._id_bus_added]
                             self._grid["ext_grid"]["vm_pu"] = val[self._id_bus_added]
-                    # ok_ind[self._id_bus_added] = False
-                    # except:
-                    #    pdb.set_trace()
                 tmp[ok_ind] = val[ok_ind]
-                # if k == "prod_v":
-                #    pass
-                #    pdb.set_trace()
             else:
                 warn = "The key {} is not recognized by PandaPowerBackend when setting injections value.".format(k)
                 warnings.warn(warn)
 
+        if np.any(redispatching != 0.):
+            # print('before tmp[ok_ind]: {}'.format(self._get_vector_inj["prod_p"](self._grid)))
+            tmp = self._get_vector_inj["prod_p"](self._grid)
+            ok_ind = np.isfinite(redispatching)
+            tmp[ok_ind] += redispatching[ok_ind]
+            # print('after tmp[ok_ind]: {}'.format(self._get_vector_inj["prod_p"](self._grid)))
+
         # topology
         # run through all substations, find the topology. If it has changed, then update it.
         beg_ = 0
@@ -376,7 +382,7 @@ class PandaPowerBackend(Backend):
         if np.any(possiblechange) or np.any(switcth_topo_vect):
             actual_topo_full = self.get_topo_vect()
             if np.any(set_topo_vect[possiblechange] != actual_topo_full[possiblechange]) or np.any(switcth_topo_vect):
-                for sub_id, nb_obj in enumerate(self.subs_elements):
+                for sub_id, nb_obj in enumerate(self.sub_info):
                     nb_obj = int(nb_obj)
                     end_ += nb_obj
                     # extract all sub information
@@ -395,11 +401,11 @@ class PandaPowerBackend(Backend):
                         actual_topo[this_topo_switch] = st
                     if np.any(this_topo_set != 0):
                         # some buses have been set
-                        sel_ = this_topo_set != 0
+                        sel_ = (this_topo_set != 0)
                         actual_topo[sel_] = this_topo_set[sel_]
 
                     # in case the topo vector is 2,2,2 etc. i convert it back to 1,1,1 etc.
-                    actual_topo = actual_topo - np.min(actual_topo) + 1
+                    actual_topo = actual_topo - np.min(actual_topo[actual_topo > 0.]) + 1
                     # implement in on the _grid
                     # change the topology in case it doesn't match the original one
                     if np.any(actual_topo != origin_topo):
@@ -407,27 +413,24 @@ class PandaPowerBackend(Backend):
                         nb_bus_now = len(np.unique(actual_topo))
                         if nb_bus_before > nb_bus_now:
                             # i must deactivate the unused bus
-                            self._grid.bus["in_service"][sub_id + self.n_substations] = False
-                            # print("I deactivated bus {} [before: {}; after {}]".format(sub_id,nb_bus_before,nb_bus_now))
+                            self._grid.bus["in_service"][sub_id + self.n_sub] = False
                         elif nb_bus_before < nb_bus_now:
                             # i must activate the new bus
-                            self._grid.bus["in_service"][sub_id + self.n_substations] = True
-                            # print("I reactivated second bus of substation {} [before: {}; after {}]".format(sub_id,nb_bus_before,nb_bus_now))
+                            self._grid.bus["in_service"][sub_id + self.n_sub] = True
 
                         # now assign the proper bus to each element
                         for i, (table, col_name, row_id) in enumerate(self._what_object_where[sub_id]):
-                            self._grid[table][col_name].iloc[row_id] = sub_id if actual_topo[i] == 1 else sub_id + self.n_substations
+                            self._grid[table][col_name].iloc[row_id] = sub_id if actual_topo[i] == 1 else sub_id + self.n_sub
+                            # if actual_topo[i] <0:
+                            #     pdb.set_trace()
+                            # self._grid[table][col_name].iloc[i] = sub_id if actual_topo[i] == 1 else sub_id + self.n_sub
 
-                            #if table == "line" or table == "trafo":
-                            #    # I connect it if it was disconnected
-                            #    self._grid[table]["in_service"].iloc[row_id] = True
-                            #    print("I reconnected a powerline")
                     beg_ += nb_obj
 
         # change line status if needed
         # note that it is a specification that lines status must override buses reconfiguration.
-        if np.any(change_status != 0.):
-            for i, el in enumerate(change_status):
+        if np.any(set_status != 0.):
+            for i, el in enumerate(set_status):
                 # TODO performance optim here, it can be vectorized
                 if el == -1:
                     self._disconnect_line(i)
@@ -446,10 +449,11 @@ class PandaPowerBackend(Backend):
                     if connected:
                         df["in_service"].iloc[tmp] = False
                     else:
-                        bus_or = set_topo_vect[self.lines_or_pos_topo_vect[i]]
-                        bus_ex = set_topo_vect[self.lines_ex_pos_topo_vect[i]]
+                        bus_or = set_topo_vect[self.line_or_pos_topo_vect[i]]
+                        bus_ex = set_topo_vect[self.line_ex_pos_topo_vect[i]]
                         if bus_ex == 0 or bus_or == 0:
-                            raise InvalidLineStatus("Line {} was disconnected. The action switched its status, without providing buses to connect it on both ends.".format(i))
+                            raise InvalidLineStatus("Line {} was disconnected. The action switched its status, "
+                                                    "without providing buses to connect it on both ends.".format(i))
                         # reconnection has then be handled in the topology
                         df["in_service"].iloc[tmp] = True
 
@@ -488,8 +492,16 @@ class PandaPowerBackend(Backend):
                     self._nb_bus_before = nb_bus
 
                 if self._grid.res_gen.isnull().values.any():
+                    # TODO see if there is a better way here
                     # sometimes pandapower does not detect divergence and put Nan.
-                    raise p.powerflow.LoadflowNotConverged
+                    raise pp.powerflow.LoadflowNotConverged
+
+                self.load_p, self.load_q, self.load_v = self._loads_info()
+                if not is_dc:
+                    if not np.all(np.isfinite(self.load_v)):
+                        # TODO see if there is a better way here
+                        # some loads are disconnected: it's a game over case!
+                        raise pp.powerflow.LoadflowNotConverged
 
                 # I retrieve the data once for the flows, so has to not re read multiple dataFrame
                 self.p_or = self._aux_get_line_info("p_from_mw", "p_hv_mw")
@@ -501,17 +513,19 @@ class PandaPowerBackend(Backend):
                 self.p_ex = self._aux_get_line_info("p_to_mw", "p_lv_mw")
                 self.q_ex = self._aux_get_line_info("q_to_mvar", "q_lv_mvar")
                 self.v_ex = self._aux_get_line_info("vm_to_pu", "vm_lv_pu")
-                self.a_ex = self._aux_get_line_info("i_to_ka", "i_lv_ka")* 1000
+                self.a_ex = self._aux_get_line_info("i_to_ka", "i_lv_ka") * 1000
                 self.a_ex[~np.isfinite(self.a_ex)] = 0.
 
                 self.v_or *= self.lines_or_pu_to_kv
                 self.v_ex *= self.lines_ex_pu_to_kv
 
+                self.prod_p, self.prod_q, self.prod_v = self._gens_info()
+
                 return self._grid.converged
 
         except pp.powerflow.LoadflowNotConverged:
             # of the powerflow has not converged, results are Nan
-            self.p_or = np.full(self.n_lines, dtype=np.float, fill_value=np.NaN)
+            self.p_or = np.full(self.n_line, dtype=np.float, fill_value=np.NaN)
             self.q_or = self.p_or
             self.v_or = self.p_or
             self.a_or = self.p_or
@@ -573,22 +587,36 @@ class PandaPowerBackend(Backend):
             self._grid.trafo["in_service"].iloc[id - self._number_true_line] = True
 
     def get_topo_vect(self):
+        # beg__ = time.time()
         # TODO refactor this, this takes a looong time
-        res = np.full(self._dim_topo, fill_value=np.NaN, dtype=np.int)
+        res = np.full(self.dim_topo, fill_value=np.NaN, dtype=np.int)
+
+        line_status = self.get_line_status()
 
         for i, (_, row) in enumerate(self._grid.line.iterrows()):
             bus_or_id = int(row["from_bus"])
             bus_ex_id = int(row["to_bus"])
-            res[self.lines_or_pos_topo_vect[i]] = 1 if bus_or_id == self.lines_or_to_subid[i] else 2
-            res[self.lines_ex_pos_topo_vect[i]] = 1 if bus_ex_id == self.lines_ex_to_subid[i] else 2
+            if line_status[i]:
+                res[self.line_or_pos_topo_vect[i]] = 1 if bus_or_id == self.line_or_to_subid[i] else 2
+                res[self.line_ex_pos_topo_vect[i]] = 1 if bus_ex_id == self.line_ex_to_subid[i] else 2
+            else:
+                res[self.line_or_pos_topo_vect[i]] = -1
+                res[self.line_ex_pos_topo_vect[i]] = -1
 
         nb = self._number_true_line
         for i, (_, row) in enumerate(self._grid.trafo.iterrows()):
             bus_or_id = int(row["hv_bus"])
             bus_ex_id = int(row["lv_bus"])
 
-            res[self.lines_or_pos_topo_vect[i + nb]] = 1 if bus_or_id == self.lines_or_to_subid[i + nb] else 2
-            res[self.lines_ex_pos_topo_vect[i + nb]] = 1 if bus_ex_id == self.lines_ex_to_subid[i + nb] else 2
+            # res[self.line_or_pos_topo_vect[i + nb]] = 1 if bus_or_id == self.line_or_to_subid[i + nb] else 2
+            # res[self.line_ex_pos_topo_vect[i + nb]] = 1 if bus_ex_id == self.line_ex_to_subid[i + nb] else 2
+            j = i + nb
+            if line_status[j]:
+                res[self.line_or_pos_topo_vect[j]] = 1 if bus_or_id == self.line_or_to_subid[j] else 2
+                res[self.line_ex_pos_topo_vect[j]] = 1 if bus_ex_id == self.line_ex_to_subid[j] else 2
+            else:
+                res[self.line_or_pos_topo_vect[j]] = -1
+                res[self.line_ex_pos_topo_vect[j]] = -1
 
         for i, (_, row) in enumerate(self._grid.gen.iterrows()):
             bus_id = int(row["bus"])
@@ -597,12 +625,13 @@ class PandaPowerBackend(Backend):
         for i, (_, row) in enumerate(self._grid.load.iterrows()):
             bus_id = int(row["bus"])
             res[self.load_pos_topo_vect[i]] = 1 if bus_id == self.load_to_subid[i] else 2
+        # self._time_topo_vect += time.time() - beg__
         return res
 
-    def generators_info(self):
-        prod_p = 1. * self._grid.res_gen["p_mw"].values
-        prod_q = 1. * self._grid.res_gen["q_mvar"].values
-        prod_v = self._grid.res_gen["vm_pu"].values * self.prod_pu_to_kv
+    def _gens_info(self):
+        prod_p = 1.0 * self._grid.res_gen["p_mw"].values
+        prod_q = 1.0 * self._grid.res_gen["q_mvar"].values
+        prod_v = 1.0 * self._grid.res_gen["vm_pu"].values * self.prod_pu_to_kv
         if self._iref_slack is not None:
             # slack bus and added generator are on same bus. I need to add power of slack bus to this one.
             if self._grid.gen["bus"].iloc[self._id_bus_added] == self.gen_to_subid[self._id_bus_added]:
@@ -610,12 +639,18 @@ class PandaPowerBackend(Backend):
                 prod_q[self._id_bus_added] += self._grid._ppc["gen"][self._iref_slack, 2]
         return prod_p, prod_q, prod_v
 
-    def loads_info(self):
+    def generators_info(self):
+        return self.prod_p, self.prod_q, self.prod_v
+
+    def _loads_info(self):
         load_p = 1. * self._grid.res_load["p_mw"].values
         load_q = 1. * self._grid.res_load["q_mvar"].values
-        load_v = self._grid.res_bus["vm_pu"].values[self._grid.load["bus"]] * self.load_pu_to_kv
+        load_v = self._grid.res_bus["vm_pu"][self.load_to_subid].values * self.load_pu_to_kv
         return load_p, load_q, load_v
 
+    def loads_info(self):
+        return self.load_p, self.load_q, self.load_v
+
     def lines_or_info(self):
         return self.p_or, self.q_or, self.v_or, self.a_or
 
@@ -623,9 +658,9 @@ class PandaPowerBackend(Backend):
         return self.p_ex, self.q_ex, self.v_ex, self.a_ex
 
     def shunt_info(self):
-        shunt_p = copy.deepcopy(self._grid.res_shunt["p_mw"].values)
-        shunt_q = copy.deepcopy(self._grid.res_shunt["q_mvar"].values)
-        shunt_v = self._grid.res_bus["vm_pu"].values[self._grid.shunt["bus"]] * self._grid.bus["vn_kv"].values[self._grid.shunt["bus"]]
+        shunt_p = 1.0 * self._grid.res_shunt["p_mw"].values
+        shunt_q = 1.0 * self._grid.res_shunt["q_mvar"].values
+        shunt_v = self._grid.res_bus["vm_pu"][self._grid.shunt["bus"]] * self._grid.bus["vn_kv"].values[self._grid.shunt["bus"]]
         shunt_bus = self._grid.shunt["bus"].values
         return shunt_p, shunt_q, shunt_v, shunt_bus
 
