@@ -9,7 +9,7 @@ import pkg_resources
 import copy
 import warnings
 
-from datetime import timedelta
+from datetime import timedelta, datetime
 import numpy as np
 import pandas as pd
 
@@ -24,6 +24,10 @@ except (ModuleNotFoundError, ImportError):
 L2RPN2019_CASEFILE = os.path.abspath(os.path.join(pkg_resources.resource_filename(__name__, "data"),
                                             "test_PandaPower", "L2RPN_2019_grid.json"))
 
+CASE_14_L2RPN2019_LAYOUT = graph_layout = [(-280, -81), (-100, -270), (366, -270), (366, -54), (-64, -54), (-64, 54),
+                                           (450, 0), (550, 0), (326, 54), (222, 108), (79, 162), (-170, 270),
+                                           (-64, 270), (222, 216)]
+
 # names of object of the grid were not in the same order as the default one
 L2RPN2019_DICT_NAMES = {'loads': {'2_C-10.61': 'load_1_0',
                                        '3_C151.15': 'load_2_1',
@@ -75,7 +79,7 @@ class ReadPypowNetData(GridStateFromFileWithForecasts):
         """
         self.n_gen = len(order_backend_prods)
         self.n_load = len(order_backend_loads)
-        self.n_lines = len(order_backend_lines)
+        self.n_line = len(order_backend_lines)
 
         self.names_chronics_to_backend = copy.deepcopy(names_chronics_to_backend)
         if self.names_chronics_to_backend is None:
@@ -143,6 +147,10 @@ class ReadPypowNetData(GridStateFromFileWithForecasts):
         self.hazards = copy.deepcopy(hazards.values[:, np.argsort(order_backend_hazards)])
         self.maintenance = copy.deepcopy(maintenance.values[:, np.argsort(order_backend_maintenance)])
 
+        # date and time
+        datetimes_ = pd.read_csv(os.path.join(self.path, "_N_datetimes{}".format(read_compressed)), sep=self.sep)
+        self.start_datetime = datetime.strptime(datetimes_.iloc[0, 0], "%Y-%b-%d")
+
         # there are maintenance and hazards only if the value in the file is not 0.
         self.maintenance = self.maintenance != 0.
         self.hazards = self.hazards != 0.
@@ -184,10 +192,10 @@ class ReadPypowNetData(GridStateFromFileWithForecasts):
         self.maintenance_forecast = copy.deepcopy(maintenance.values[:, np.argsort(order_backend_maintenance)])
 
         # there are maintenance and hazards only if the value in the file is not 0.
-        self.maintenance_time = np.zeros(shape=(self.load_p.shape[0], self.n_lines), dtype=np.int) - 1
-        self.maintenance_duration = np.zeros(shape=(self.load_p.shape[0], self.n_lines), dtype=np.int)
-        self.hazard_duration = np.zeros(shape=(self.load_p.shape[0], self.n_lines), dtype=np.int)
-        for line_id in range(self.n_lines):
+        self.maintenance_time = np.zeros(shape=(self.load_p.shape[0], self.n_line), dtype=np.int) - 1
+        self.maintenance_duration = np.zeros(shape=(self.load_p.shape[0], self.n_line), dtype=np.int)
+        self.hazard_duration = np.zeros(shape=(self.load_p.shape[0], self.n_line), dtype=np.int)
+        for line_id in range(self.n_line):
             self.maintenance_time[:, line_id] = self.get_maintenance_time_1d(self.maintenance[:, line_id])
             self.maintenance_duration[:, line_id] = self.get_maintenance_duration_1d(self.maintenance[:, line_id])
             self.hazard_duration[:, line_id] = self.get_maintenance_duration_1d(self.hazards[:, line_id])
@@ -210,18 +218,12 @@ class L2RPN2019_Action(Action):
     **NB** This class doesn't allow to connect object to other buses than their original bus. In this case,
     reconnecting a powerline cannot be considered "ambiguous". We have to
     """
-    def __init__(self, n_gen, n_load, n_lines, subs_info, dim_topo,
-                 load_to_subid, gen_to_subid, lines_or_to_subid, lines_ex_to_subid,
-                 load_to_sub_pos, gen_to_sub_pos, lines_or_to_sub_pos, lines_ex_to_sub_pos,
-                 load_pos_topo_vect, gen_pos_topo_vect, lines_or_pos_topo_vect, lines_ex_pos_topo_vect):
+    def __init__(self, gridobj):
         """
         See the definition of :func:`Action.__init__` and of :class:`Action` for more information. Nothing more is done
         in this constructor.
         """
-        Action.__init__(self, n_gen, n_load, n_lines, subs_info, dim_topo,
-                 load_to_subid, gen_to_subid, lines_or_to_subid, lines_ex_to_subid,
-                 load_to_sub_pos, gen_to_sub_pos, lines_or_to_sub_pos, lines_ex_to_sub_pos,
-                 load_pos_topo_vect, gen_pos_topo_vect, lines_or_pos_topo_vect, lines_ex_pos_topo_vect)
+        Action.__init__(self, gridobj)
 
         # the injection keys is not authorized, meaning it will send a warning is someone try to implement some
         # modification injection.
@@ -300,7 +302,7 @@ class L2RPN2019_Action(Action):
         size: ``int``
             The size of :class:`PowerLineSet` converted to an array.
         """
-        return self._n_lines + self._dim_topo
+        return self.n_line + self.dim_topo
 
     def to_vect(self):
         """
@@ -311,7 +313,7 @@ class L2RPN2019_Action(Action):
 
         Returns
         -------
-        as_vect: :class:`numpy.array`, dtype:float
+        _vectorized: :class:`numpy.array`, dtype:float
             The instance of this action converted to a vector.
         """
         if self.as_vect is None:
@@ -345,10 +347,10 @@ class L2RPN2019_Action(Action):
         if vect.shape[0] != self.size():
             raise IncorrectNumberOfElements("Incorrect number of elements found while loading a \"TopologyAction\" from a vector. Found {} elements instead of {}".format(vect.shape[1], self.size()))
         prev_ = 0
-        next_ = self._n_lines
+        next_ = self.n_line
 
         self._switch_line_status = vect[prev_:next_]
-        self._switch_line_status = self._switch_line_status.astype(np.bool); prev_=next_; next_+= self._dim_topo
+        self._switch_line_status = self._switch_line_status.astype(np.bool); prev_=next_; next_+= self.dim_topo
         self._change_bus_vect = vect[prev_:next_]
         self._change_bus_vect = self._change_bus_vect.astype(np.bool)
 
@@ -367,16 +369,20 @@ class L2RPN2019_Action(Action):
     #     """
     #     sel_ = self._set_line_status == 1
     #     if np.any(sel_):
-    #         self._set_topo_vect[self._lines_ex_pos_topo_vect[sel_]] = 1
-    #         self._set_topo_vect[self._lines_or_pos_topo_vect[sel_]] = 1
+    #         self._set_topo_vect[self.line_ex_pos_topo_vect[sel_]] = 1
+    #         self._set_topo_vect[self.line_or_pos_topo_vect[sel_]] = 1
 
-    def sample(self):
+    def sample(self, space_prng):
         """
         Sample a PowerlineSwitch Action.
 
         By default, this sampling will act on one random powerline, and it will either
         disconnect it or reconnect it each with equal probability.
 
+        Parameters
+        ----------
+        space_prng
+
         Returns
         -------
         res: :class:`PowerLineSwitch`
@@ -387,6 +393,6 @@ class L2RPN2019_Action(Action):
         val = 2*np.random.randint(0, 2) - 1  # the action: +1 reconnect it, -1 disconnect it
         self._set_line_status[i] = val
         if val == 1:
-            self._set_topo_vect[self._lines_ex_pos_topo_vect[i]] = 1
-            self._set_topo_vect[self._lines_or_pos_topo_vect[i]] = 1
+            self._set_topo_vect[self.line_ex_pos_topo_vect[i]] = 1
+            self._set_topo_vect[self.line_or_pos_topo_vect[i]] = 1
         return self
