@@ -6,12 +6,14 @@ Document will be made later on.
 import os
 import pkg_resources
 
-__version__ = '0.4.1'
+__version__ = '0.5.7'
 
 __all__ = ['Action', "BackendPandaPower", "Agent", "Backend", "ChronicsHandler", "Environment", "Exceptions",
-           "Observation", "Parameters", "GameRules", "Reward", "Runner", "main", "Utils", "PlotPlotly"]
+           "Observation", "Parameters", "GameRules", "Reward", "Runner", "main", "Utils", "PlotPlotly", "EpisodeData",
+           "DownloadDataset", "PlotMatplotlib"]
 
 from .MakeEnv import make
+
 # reference case 14
 from .MakeEnv import CASE_14_FILE, CHRONICS_FODLER, CHRONICS_MLUTIEPISODE, NAMES_CHRONICS_TO_BACKEND
 # small example
